/*
 * $Id: voct.c 5720 2017-11-07 19:30:36Z mskala $
 *
 * V/octave tracking test program
 * Copyright (C) 2017  Matthew Skala
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Matthew Skala
 * https://northcoastsynthesis.com/
 * mskala@northcoastsynthesis.com
 */

#include <string.h>
#include <unistd.h>
#include <math.h>

#include <alsa/asoundlib.h>

double x[316],y[316];

void do_regression(int j,int points) {
   int i;
   double sx=0.0,sy=0.0,sxy=0.0,sxx=0.0,syy=0.0,a;

   if (j<points)
     return;

   for (i=(j-points+1);i<=j;i++) {
      sx+=x[i%316];
      sy+=y[i%316];
      sxx+=(x[i%316]*x[i%316]);
      sxy+=(x[i%316]*y[i%316]);
      syy+=(y[i%316]*y[i%316]);
   }

   a=(((double)points)*sxy-sx*sy)/(((double)points)*sxx-sx*sx);
   printf("\t%f",1.0/a);
}

int main(int argc,char **argv) {
   snd_rawmidi_t *midi_out;
   int note,i,j,k;
   char out_buf[3];
   FILE *wav_file;
   unsigned char wav_data[44+192000*3];
   int state,first_fall,last_fall,num_falls;
   double freq,octaves;

   if (snd_rawmidi_open(NULL,&midi_out,"hw:1,0,0",
			SND_RAWMIDI_NONBLOCK|SND_RAWMIDI_APPEND)<0) {
      return 1;
   }
  
   j=0;
   while (1) {
      note=36+(rand()%49);
      
      j++;
      printf("%d %d ",j,note);

      usleep(100000);

      out_buf[0]=0x90;
      out_buf[1]=note;
      out_buf[2]=100;
      snd_rawmidi_write(midi_out,out_buf,3);
      
      usleep(100000);

      out_buf[0]=0x90;
      out_buf[1]=note;
      out_buf[2]=0;
      snd_rawmidi_write(midi_out,out_buf,3);
      
      usleep(100000);

      system("arecord -c1 -r192000 /tmp/test.wav -d3 2> /dev/null");
      
      usleep(100000);

      wav_file=fopen("/tmp/test.wav","r");
      fread(wav_data,1,44+192000*3,wav_file);
      fclose(wav_file);
      
      state=0;
      first_fall=0;
      last_fall=0;
      num_falls=0;
      
      for (i=44+192000/2;i<44+480000;i++) {
	 if ((state!=1) && (wav_data[i]<0x7F)) {
	    if (state==2) {
	       if (num_falls==0)
		 first_fall=i;
	       num_falls++;
	       last_fall=i;
	    }
	    state=1;
	 } else if ((state==1) && (wav_data[i]>0x81))
	      state=2;
      }
      
      if ((num_falls>2) && (num_falls<60000)) {
	 freq=((double)(num_falls-1))*192000.0/((double)(last_fall-first_fall));
	 octaves=log(freq/440.0)/log(2.0);
	 printf("%f %f",freq,octaves);
	 x[j%316]=(((double)note)-69.0)/12.0;
	 y[j%316]=octaves;
	 
	 do_regression(j,10);
	 do_regression(j,32);
	 do_regression(j,100);
	 do_regression(j,316);
	 putchar('\n');
      } else {
	 puts("--");
	 j--;
      }
      fflush(stdout);
   }
}
